/*
 * RTSP (Real Time Streaming Protocol) Reference Implementation
 * 
 *     RTSP message parsing routines
 *
 *  Copyright (C) 1996 Progressive Networks.
 * 
 *  This program is free software; you can only redistribute it and/or
 *  modify it under the terms of version 2 of the GNU General Public
 *  License as published by the Free Software Foundation.  A copy of this
 *  license may be found in the "LICENSE" directory of this distribution.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY OR REPRESENTATION AS TO OWNERSHIP OR
 *  NON-INFRINGEMENT; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 *  License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 *  Please submit patches, modified versions and derivative works to
 *  Progressive Networks at "rtsp-feedback@prognet.com" or write to
 *  Progressive Networks, 1111 Third Ave. Ste 2900, Seattle, WA 98101, USA.
 *  We will do our best to incorporate your enhancements into the core
 *  distribution.
 *
 *  For more information about this program, please visit the following
 *  URL:
 *     http://www.real.com/prognet/rt 
 *
 *
 * $Id: parse.h,v 1.6 1997/04/01 17:30:53 robla Exp $
 *
 */

#ifndef _PARSE_H_
#define _PARSE_H_

#include <socket.h>

#ifdef __cplusplus
extern "C"
{
#endif

#define HDRBUFLEN   4096
#define TOKENLEN    100

#define SETUP_TKN       "SETUP"
#define REDIRECT_TKN    "REDIRECT"
#define PLAY_TKN        "PLAY"
#define PAUSE_TKN       "PAUSE"
#define SESSION_TKN     "SESSION"
#define RECORD_TKN      "RECORD"
#define EXT_METHOD_TKN  "EXT-"

/*  methods that have changed between drafts of the RTSP document */
#ifdef DRAFT02
#define HELLO_TKN       "OPTIONS"
#define GET_TKN         "DESCRIBE"
#define GET_PARAM_TKN   "GET_PARAMETER"
#define SET_PARAM_TKN   "SET_PARAMETER"
#define CLOSE_TKN       "TEARDOWN"
#else /* DRAFT01 (roughly; compatible with RealMedia beta 1) */
#define HELLO_TKN       "HELLO"
#define GET_TKN         "GET"
#define GET_PARAM_TKN   "GET_PARAM"
#define SET_PARAM_TKN   "SET_PARAM"
#define CLOSE_TKN       "STOP" 
#endif /* DRAFT02 | DRAFT01 */


/* method codes */
#define RTSP_SETUP_METHOD     0
#define RTSP_GET_METHOD       1
#define RTSP_REDIRECT_METHOD  2
#define RTSP_PLAY_METHOD      3
#define RTSP_PAUSE_METHOD     4
#define RTSP_SESSION_METHOD   5
#define RTSP_HELLO_METHOD     6
#define RTSP_RECORD_METHOD    7
#define RTSP_CLOSE_METHOD     8
#define RTSP_GET_PARAM_METHOD 9
#define RTSP_SET_PARAM_METHOD 10
#define RTSP_EXTENSION_METHOD 11

/*
 * method response codes.  These are 100 greater than their
 * associated method values.  This allows for simplified
 * creation of event codes that get used in event_handler()
 */
#define RTSP_SETUP_RESPONSE      100
#define RTSP_GET_RESPONSE        101
#define RTSP_REDIRECT_RESPONSE   102
#define RTSP_PLAY_RESPONSE       103
#define RTSP_PAUSE_RESPONSE      104
#define RTSP_SESSION_RESPONSE    105
#define RTSP_HELLO_RESPONSE      106
#define RTSP_RECORD_RESPONSE     107
#define RTSP_CLOSE_RESPONSE      108
#define RTSP_GET_PARAM_RESPONSE  109
#define RTSP_SET_PARAM_RESPONSE  110
#define RTSP_EXTENSION_RESPONSE  111

typedef struct tokens
{
   char  *token;
   int   code;
} TKN;

char *get_stat( int code );
char * get_method( int code );
void remove_msg( int len );
void discard_msg( void );
int valid_response_msg( char *s, u_short *seq_num, u_short *status,
                        char *msg );
int validate_method( char *s );
void get_msg_len( int *hdr_len, int *body_len );
char *get_int( char *buf, int hlen, char *tkn, char *sep, int *nint, long *nlong );
char *get_string( char *buf, int hlen, char *tkn, char *sep );
void chk_buff_size( int len );
void str_to_lower( char *s, int l );
int set_stream_settings( char *b, int len, struct SESSION_STATE *state );
void free_streams( struct SESSION_STATE *state );

#ifdef _cplusplus
}
#endif

#endif  /* _PARSE_H_ */
