/*
 * RTSP (Real Time Streaming Protocol) Reference Implementation
 * 
 *     Scheduling routines
 *
 *  Copyright (C) 1996 Progressive Networks.
 * 
 *  This program is free software; you can only redistribute it and/or
 *  modify it under the terms of version 2 of the GNU General Public
 *  License as published by the Free Software Foundation.  A copy of this
 *  license may be found in the "LICENSE" directory of this distribution.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY OR REPRESENTATION AS TO OWNERSHIP OR
 *  NON-INFRINGEMENT; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 *  License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 *  Please submit patches, modified versions and derivative works to
 *  Progressive Networks at "rtsp-feedback@prognet.com" or write to
 *  Progressive Networks, 1111 Third Ave. Ste 2900, Seattle, WA 98101, USA.
 *  We will do our best to incorporate your enhancements into the core
 *  distribution.
 *
 *  For more information about this program, please visit the following
 *  URL:
 *     http://www.real.com/prognet/rt 
 *

 *
 * $Id: scheduler.c,v 1.14 1997/04/01 17:30:55 robla Exp $
 *
 */

#include <assert.h>
#include <stdlib.h>
#include <stdio.h>

#include "scheduler.h"

#define MAXSCHEDULE 20
#define INVALID_SCHEDULE_ID ((u_long)-1)

static u_long next_schedule_id = 0;

static struct SCHEDULE_LIST
{
    u_long id;
    u_long time;
    u_long session_id;
    void (*fn)(void *);
    void *arg;
} schedule_list[MAXSCHEDULE];

void
schedule_init()
{
    int i;

    for(i=0;i<MAXSCHEDULE;i++)
        schedule_list[i].id = INVALID_SCHEDULE_ID;
}

u_long 
schedule_enter(u_long time, u_long session_id, void (*fn)(void*), void* arg)
{
    int i;

    for(i=0;i<MAXSCHEDULE;i++)
    {
        if(schedule_list[i].id == INVALID_SCHEDULE_ID)
        {
            schedule_list[i].id = ++next_schedule_id;
            schedule_list[i].time = time;
            schedule_list[i].session_id = session_id;
            schedule_list[i].fn = fn;
            schedule_list[i].arg = arg;
            return schedule_list[i].id;
	}
    }
    return INVALID_SCHEDULE_ID;
}

void
schedule_remove(u_long id)
{
    int i;

    for(i=0;i<MAXSCHEDULE;i++)
    {
        if(schedule_list[i].id == id)
        {
            schedule_list[i].id = INVALID_SCHEDULE_ID;
            return;
	}
    }
}

void
schedule_remove_session(u_long session_id)
{
    int i;

    for(i=0;i<MAXSCHEDULE;i++)
    {
        if(schedule_list[i].session_id == session_id)
            schedule_list[i].id = INVALID_SCHEDULE_ID;
    }
}

void
schedule_execute(u_long time)
{

   int i;

   for(i=0;i<MAXSCHEDULE;i++)
   {
      if((schedule_list[i].id != INVALID_SCHEDULE_ID) && 
         (schedule_list[i].time <= time))
      {
         schedule_list[i].fn(schedule_list[i].arg);
         schedule_list[i].id = INVALID_SCHEDULE_ID;
      }
   }
}

int
schedule_check( u_long session_id )
{
   int   i;
   
   for( i = 0; i < MAXSCHEDULE; i++ )
      if (schedule_list[i].session_id == session_id)
         return( 1 );
   return( 0 );
}