/*
 * RTSP (Real Time Streaming Protocol) Reference Implementation
 *
 *	Misc. utilities
 *
 *
 *  Copyright (C) 1996 Progressive Networks.
 * 
 *  This program is free software; you can only redistribute it and/or
 *  modify it under the terms of version 2 of the GNU General Public
 *  License as published by the Free Software Foundation.  A copy of this
 *  license may be found in the "LICENSE" directory of this distribution.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY OR REPRESENTATION AS TO OWNERSHIP OR
 *  NON-INFRINGEMENT; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 *  License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 *  Please submit patches, modified versions and derivative works to
 *  Progressive Networks at "rtsp-feedback@prognet.com" or write to
 *  Progressive Networks, 1111 Third Ave. Ste 2900, Seattle, WA 98101, USA.
 *  We will do our best to incorporate your enhancements into the core
 *  distribution.
 *
 *  For more information about this program, please visit the following
 *  URL:
 *     http://www.real.com/prognet/rt 
 *

 *
 * $Id: util.c,v 1.20 1997/05/30 04:53:50 robla Exp $
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#ifdef WIN32
#include <io.h>
#endif
#if defined sun
#include <strings.h>
#else
#include <string.h>
#endif
#include "machdefs.h"
#include "rtsp.h"
#include "messages.h"
#include "util.h"


/* controlled from player/server */
int debug_toggle = 0;


void
dump_buffer(char *buffer, int n)
{
   int   i;
   int   c = 0;

   printf( "Hex Dump:\n" );   
   for (i = 0; i < n; i++)
   {
      printf ("%02x ", (unsigned char)buffer[i]);
      c += 3;
      if (c > 77)
      {
         printf ("\n");
         c = 0;
      }
   }
   printf ("\n");

   if ( isprint((int) *buffer) || isspace((int) *buffer) )
   {
      printf( "ASCII Dump\n" );
      while ( n-- && (isprint((int) *buffer) || isspace((int) *buffer)) )
         printf( "%c", *buffer++ );
      printf( "\n" );
   }
}

int
parse_url(const char *url, char *server, u_short *port, char *file_name)
{
    /* expects format '[rtsp://server[:port/]]filename' */

   int valid_url = 0;
   /* copy url */
   char *full = malloc(strlen(url) + 1);
   strcpy(full, url);
   if(strncmp(full,"rtsp://", 7) == 0)
   {
      char *token;
      int has_port = 0;
      if(strchr(&full[7], ':'))
         has_port = 1;
      token = strtok(&full[7], " :/\t\n");
      if(token)
   	{
         strcpy(server, token);
         if(has_port)
         {
            char *port_str = strtok(&full[strlen(server)+7+1], " /\t\n");
            if(port_str)
               *port = (u_short)atol(port_str);
            else
               *port = RTSP_DEFAULT_PORT;
         }
         else
            *port = RTSP_DEFAULT_PORT;
         /* don't requre a file name */
         valid_url = 1;
         token = strtok(NULL, " ");
         if(token)
            strcpy(file_name, token);
         else
            file_name[0] = '\0';
      }
   }
   else
   {
      /* try just to extract a file name */
      char *token = strtok(full, " \t\n");
      if(token)
      {
         strcpy(file_name, token);
         server[0] = '\0';
         valid_url = 1;
      }
   }
   free(full);
   return valid_url;
}

void
free_streams( struct SESSION_STATE *state )
{
   STREAM   *s;         /* stream descriptor pointer */
   STREAM   *n;
   
   for ( s = state->streams; s; s = n )
   {
      n = s->next;
      if ( s->stream_fd )
         close( s->stream_fd );
      free( s );
   }
   state->streams = 0;
}

int big_endian()
{
	short test = 0x00ff;
	char* p = (char*)&test;
	return (p[0] == 0);
}

void swap_word(u_int16* pWord, int nWords)
{
	int i;
	u_int16 tmp;
	char* pTmp = (char*)&tmp;

	for(i=0;i<nWords;++i)
	{
		char* pData = (char*)&pWord[i];

		pTmp[0] = pData[1];
		pTmp[1] = pData[0];
		pWord[i] = tmp;
	}
}

void swap_dword(u_int32* pDWord, int nDWords)
{     /* not expected to work with Middle-Endian systems. */
	int i;
	u_int32 tmp;
	char* pTmp = (char*)&tmp;

	for(i=0;i<nDWords;++i)
	{
		char* pData = (char*)&pDWord[i];

		pTmp[0] = pData[3];
		pTmp[1] = pData[2];
		pTmp[2] = pData[1];
		pTmp[3] = pData[0];
		pDWord[i] = tmp;
	}
}
