/*
 * RTSP (Real Time Streaming Protocol) Reference Implementation
 *
 *     Tcl/Tk GUI Interface
 *
 *
 *  Copyright (C) 1996 Progressive Networks.
 * 
 *  This program is free software; you can only redistribute it and/or
 *  modify it under the terms of version 2 of the GNU General Public
 *  License as published by the Free Software Foundation.  A copy of this
 *  license may be found in the "LICENSE" directory of this distribution.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY OR REPRESENTATION AS TO OWNERSHIP OR
 *  NON-INFRINGEMENT; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 *  License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 *  Please submit patches, modified versions and derivative works to
 *  Progressive Networks at "rtsp-feedback@prognet.com" or write to
 *  Progressive Networks, 1111 Third Ave. Ste 2900, Seattle, WA 98101, USA.
 *  We will do our best to incorporate your enhancements into the core
 *  distribution.
 *
 *  For more information about this program, please visit the following
 *  URL:
 *     http://www.real.com/prognet/rt 
 *

 *
 * $Id: gui.c,v 1.24 1997/05/30 04:53:56 robla Exp $
 *
 */

#if defined AUDIO
#include <sys/soundcard.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <tcl.h>
#include <tk.h>

#include "machdefs.h"
#include "eventloop.h"
#include "player.h"
#include "../common/session.h"


Tcl_Interp *interp;
char url[1024] = { 0 };
static int wav_session = 0;
static int wav_fd = 0;
#if defined AUDIO
static int wav_rate = 0;
static int wav_sampleformat = 0;
#endif
static int hello_yet = 0;


int
event()
{
    eventloop();
    Tk_CreateTimerHandler(10, (Tk_TimerProc *)event, 0);

    return TCL_OK;
}


void interface_new_session(u_long id)
{
    char buf[1024];

    sprintf(buf, "NewSession %ld", id);
    Tcl_Eval(interp, buf);
}

void interface_close_session(u_long id)
{
    char buf[1024];

    sprintf(buf, "destroy .ns%ld", id);
    Tcl_Eval(interp, buf);

    close (wav_fd);
    wav_fd = 0;
    wav_session = 0;
}

void interface_can_play(u_long id)
{
    /* only used in Win32 version */
}

int
url_cb(ClientData *cd, Tcl_Interp *interp, int argc, char **argv)
{
    char buf[1024];

    strcpy(url, argv[1]);
    

    if (hello_yet)
    {
      sprintf(buf, "get %s\n", url);
      handle_command(buf);
    }
    else 
    {
      sprintf(buf, "open %s\n", url);
      handle_command(buf);
      sprintf(buf, "get %s\n", url);
      handle_command(buf);
    }
    return TCL_OK;
}


int
play_cb(ClientData *cd, Tcl_Interp *interp, int argc, char **argv)
{
    char buf[1024];

    sprintf(buf, "play %s", argv[1]);
    handle_command(buf);
    if (atoi(argv[2]))
    {
    	if (!wav_session)
	    wav_session = atoi(argv[1]);
    	else
    	{
    	    sprintf(buf, ".ns%d.debug insert end \"Audio device in use, using streaming text mode\n\"", atoi(argv[1]));
    	    Tcl_Eval(interp, buf);
    	}
    }

    return TCL_OK;
}


int
stop_cb(ClientData *cd, Tcl_Interp *interp, int argc, char **argv)
{
    char buf[1024];

    sprintf(buf, "close %s", argv[1]);
    handle_command(buf);

    return TCL_OK;
}


int
resume_cb(ClientData *cd, Tcl_Interp *interp, int argc, char **argv)
{
    char buf[1024];

    sprintf(buf, "play %s", argv[1]);
    handle_command(buf);

    return TCL_OK;
}

int
speed_cb(ClientData *cd, Tcl_Interp *interp, int argc, char **argv)
{
#if defined AUDIO
    char buf[1024];
    int x;

    x = 65536 + (atoi(argv[2]) - 500) * 130;

    sprintf(buf, "z %s %d", argv[1], x);
    handle_command(buf);

    if (wav_session == atoi(argv[1]))
    {
    	x = wav_rate * (x / 65536.0);
	ioctl(wav_fd, SNDCTL_DSP_SPEED, &x);
    }
#endif
    return TCL_OK;
}

int
quit_cb(ClientData *cd, Tcl_Interp *interp, int argc, char **argv)
{
    handle_command("quit\n");

    return TCL_OK;
}

void
interface_hello()
{
    char buf[1024];

    hello_yet++;
    sprintf(buf, ".debug insert end \"Got HELLO\"\n");
    Tcl_Eval(interp, buf);

    if (url[0])
    {
	sprintf(buf, "open %s\n", url);
	handle_command(buf);
    }
}

#ifdef AUDIO
void
convert16to8(u_short n, char *data)
{
  int i;

  for (i=0;i<=(n/2);i++) {
    data[i]=data[i*2]-128;
  }
}
#endif

void
interface_stream_output(u_long id, u_short n, char *data)
{
    char buf[1024];

#ifdef AUDIO
    /* XXX - Packet size must be large enough to transmit the whole header */
    /* XXX - We could really use some buffering :-) */

    if (wav_session == id) {
    	if (!wav_fd)
    	{
    	    int x;

    	    wav_fd = open("/dev/dsp", O_WRONLY);
    	    wav_rate = PCM_hdr.nSamplesPerSec;
            if(PCM_hdr.wBitsPerSample==16) {
              x = AFMT_S16_BE;
	      if(ioctl(wav_fd, SNDCTL_DSP_SETFMT, &x)==-1)
              {
                 printf("Error: Cannot set sample format\n");
		 exit(1);
              }
	      wav_sampleformat = x; 
              printf("Sample Format (supported by card): %d\n", x);
	    }
	    else if (PCM_hdr.wFormatTag==0x7)
	    {
	      /*  muLaw  */
	      x = AFMT_MU_LAW;
	      if(ioctl(wav_fd, SNDCTL_DSP_SETFMT, &x)==-1)
              {
                 printf("Error: Cannot set sample format\n");
		 exit(1);
              }
	      wav_sampleformat = x; 
	    }
	    else
	    {
	      /*  assume unsigned 8-bit, and that /dev/dsp is all setup */
	      wav_sampleformat = AFMT_U8;
	    }
	      
    	    x = PCM_hdr.nChannels -1;	
	    if (ioctl(wav_fd, SNDCTL_DSP_STEREO, &x)==-1) {
                 printf("Error: Cannot set channels\n");
		 exit(1);
	    }
    	    x = wav_rate;		
	    if (ioctl(wav_fd, SNDCTL_DSP_SPEED, &x)==-1) {
                 printf("Error: Cannot set speed\n");
	    }

	    sprintf(buf, ".ns%ld.output insert end \"WAV File\n\"", id);
    	    Tcl_Eval(interp, buf);
	    sprintf(buf, ".ns%ld.output insert end \"Frequency: %ld\n\"", 
		    id, PCM_hdr.nSamplesPerSec);
    	    Tcl_Eval(interp, buf);
	    sprintf(buf, ".ns%ld.output insert end \"Channels: %d\n\"", 
		    id, PCM_hdr.nChannels);
    	    Tcl_Eval(interp, buf);
	    sprintf(buf, ".ns%ld.output insert end \"Bits per sample: %d\n\"", 
		    id, PCM_hdr.wBitsPerSample);
    	    Tcl_Eval(interp, buf);
    	}
	if(PCM_hdr.wBitsPerSample==16 && wav_sampleformat == AFMT_U8 ) 
        {
	  convert16to8(n, data);
	  write (wav_fd, data, (n/2));
	}
	else
        {
	  write (wav_fd, data, n);
	}
    	return;
    }
#endif
#ifndef AUDIO
    u_short i;

    sprintf(buf, ".ns%ld.output insert end \"", id);
    for (i = 0; i < n; i++)
    {
    	if (data[i] != '\n')
    	{
    	    int x = strlen(buf);
    	    buf[x] = data[i];
    	    buf[x+1] = 0;
    	}
    	else
    	{
    	    strcat(buf, "\n\"\n");
	    Tcl_Eval(interp, buf);
	    sprintf(buf, ".ns%ld.output insert end \"", id);
    	}
    }
    strcat(buf, "\"\n");
    Tcl_Eval(interp, buf);

    sprintf(buf, ".ns%ld.output see end\n", id);
    Tcl_Eval(interp, buf);
#endif
}

void
interface_init(void *p)
{
    interp = Tcl_CreateInterp();

    if (Tcl_Init(interp) == TCL_ERROR)
    {
	fprintf (stderr, "Fatal: Tcl_Init Failed\n");
	exit (1);
    }

#ifdef TK40
    Tk_CreateMainWindow(interp, 0, "rtspref", "Rtspref");
#endif

    if (Tk_Init(interp) == TCL_ERROR)
    {
	fprintf (stderr, "Fatal: Tk_Init Failed\nCheck Your DISPLAY\n");
	exit (1);
    }

    Tcl_EvalFile(interp, "gui.tcl");
    Tcl_CreateCommand(interp, (char *)"url_cb", (Tcl_CmdProc *)url_cb, 0, 0);
    Tcl_CreateCommand(interp, (char *)"Play", (Tcl_CmdProc *)play_cb, 0, 0);
    Tcl_CreateCommand(interp, (char *)"Stop", (Tcl_CmdProc *)stop_cb, 0, 0);
    Tcl_CreateCommand(interp, (char *)"Resume", (Tcl_CmdProc *)resume_cb, 0, 0);
    Tcl_CreateCommand(interp, (char *)"Speed", (Tcl_CmdProc *)speed_cb, 0, 0);
    Tcl_CreateCommand(interp, (char *)"Quit", (Tcl_CmdProc *)quit_cb, 0, 0);
    Tk_CreateTimerHandler(10, (Tk_TimerProc *)event, 0);
    Tk_MainLoop();
}
void
interface_error( const char *err )
{
  /* used only in text.c */
}

void
interface_alert_msg( const char *msg )
{
  /* used only in text.c */
}

void interface_renew_play( u_long id )
{
  /* used only in text.c */
}

void interface_close_stream(int id)
{
  /* used only in text.c */
}

void interface_start_play( u_long id )
{
  /* used only in text.c */
}

void
interface_stream_done( u_long sessionId )
{
#ifdef AUDIO
    close (wav_fd);
    wav_fd = 0;
    wav_session = 0;
#endif  /* AUDIO */
}
