/*
 * RTSP (Real Time Streaming Protocol) Reference Implementation
 *
 *     ASCII Interface
 *
 *
 *  Copyright (C) 1996 Progressive Networks.
 * 
 *  This program is free software; you can only redistribute it and/or
 *  modify it under the terms of version 2 of the GNU General Public
 *  License as published by the Free Software Foundation.  A copy of this
 *  license may be found in the "LICENSE" directory of this distribution.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY OR REPRESENTATION AS TO OWNERSHIP OR
 *  NON-INFRINGEMENT; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 *  License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 *  Please submit patches, modified versions and derivative works to
 *  Progressive Networks at "rtsp-feedback@prognet.com" or write to
 *  Progressive Networks, 1111 Third Ave. Ste 2900, Seattle, WA 98101, USA.
 *  We will do our best to incorporate your enhancements into the core
 *  distribution.
 *
 *  For more information about this program, please visit the following
 *  URL:
 *     http://www.real.com/prognet/rt 
 *

 *
 * $Id: text.c,v 1.21 1997/05/30 04:53:59 robla Exp $
 *
 */

#if defined AUDIO
#ifdef __FreeBSD__
#include <machine/soundcard.h>
#else
#include <sys/soundcard.h>
#endif
#endif

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <fcntl.h>
#include "machdefs.h"
#include "socket.h"
#include "../common/session.h"

#if defined AUDIO
/*  nasty globals  */
static int wav_session = 0;
static int wav_fd = 0;
static int wav_rate = 0;
static int wav_sampleformat = 0;
#endif

int errno = 0;
/*  end of nasty globals */


void interface_init()
{
}

void interface_can_play(int id)
{
    printf("Can play %d\n", id);
}

void interface_close_stream(int id)
{
    printf( "CLOSE\n" );
}

void interface_new_session(int id)
{
    printf("New session: %d\n", id);
}

void interface_close_session(int id)
{
    printf("Close session: %d\n", id);
}

void interface_hello()
{
    printf ("HELLO\n");
}

void interface_renew_play( u_long id )
{
   printf( "Enter new command\n" );
}

void interface_start_play( u_long id )
{
   printf( "PLAY started.\n" );
}

#ifdef AUDIO
void
convert16to8(u_short n, char *data)
{
  int i;

  for (i=0;i<=(n/2);i++) {
    data[i]=data[i*2]-128;
  }
}

void
interface_stream_output(u_long id, u_short n, char *data)
{
    if (wav_session == id) {
    	if (!wav_fd)
    	{
    	    int x;

    	    wav_fd = open("/dev/dsp", O_WRONLY);
    	    wav_rate = PCM_hdr.nSamplesPerSec;
            if(PCM_hdr.wBitsPerSample==16) {
              x = AFMT_S16_BE;
	      if(ioctl(wav_fd, SNDCTL_DSP_SETFMT, &x)==-1)
              {
                 printf("Error: Cannot set sample format\n");
		 exit(1);
              }
	      wav_sampleformat = x; 
              printf("Sample Format (supported by card): %d\n", x);
	    }
	    else if (PCM_hdr.wFormatTag==0x7)
	    {
	      /*  muLaw  */
	      x = AFMT_MU_LAW;
	      if(ioctl(wav_fd, SNDCTL_DSP_SETFMT, &x)==-1)
              {
                 printf("Error: Cannot set sample format\n");
		 exit(1);
              }
	      wav_sampleformat = x; 
	    }
	    else
	    {
	      /*  assume unsigned 8-bit, and that /dev/dsp is all setup */
	      wav_sampleformat = AFMT_U8;
	    }
	      
    	    x = PCM_hdr.nChannels -1;	
	    if (ioctl(wav_fd, SNDCTL_DSP_STEREO, &x)==-1) {
                 printf("Error: Cannot set channels\n");
		 exit(1);
	    }
    	    x = wav_rate;		
	    if (ioctl(wav_fd, SNDCTL_DSP_SPEED, &x)==-1) {
                 printf("Error: Cannot set speed\n");
	    }

	    printf("WAV File\n");
	    printf("Frequency: %ld\n", PCM_hdr.nSamplesPerSec);
	    printf("Channels: %d\n", PCM_hdr.nChannels);
            printf("Bits Per Sample: %d\n", PCM_hdr.wBitsPerSample);
    	}
	
	if(PCM_hdr.wBitsPerSample==16 && wav_sampleformat == AFMT_U8 ) 
        {
	  convert16to8(n, data);
	  write (wav_fd, data, (n/2));
	}
	else
        {
	  write (wav_fd, data, n);
	}
    	return;
    }
}
#endif /* AUDIO */

#ifndef AUDIO
void
interface_stream_output(u_long id, u_short n, const char *data)
{
   static int  created = 0;
   int         fd;
   char        filename[256];
   int         flags;
   int         cnt;
   
   /* append output to file named 'audioXXXX.txt' */
   sprintf(filename, "audio%ld.dat", id);
   if ( created )
      flags = O_WRONLY | O_APPEND;
   else
   {
      flags = O_CREAT | O_WRONLY | O_TRUNC;
      created = 1;
   }

   fd = open( filename, flags, 0x1a4 );  /* rw/r/r permissions */
   if ( fd == -1 )
   {
      printf( "ALERT:  error opening file interface_stream_output(). "
              "errno=%d", errno );
      return;
   }
   cnt = write( fd, data, n );
   if ( cnt == -1 )
      printf( "ALERT: error writing interface_stream_output(). "
              "errno=%d", errno );
   else if ( cnt != n )
      printf( " Error write %d byte(s) to %s. Only wrote %d byte(s).",
              n, filename, cnt );
   close( fd );
}
#endif  /* ifndef AUDIO */

void
interface_stream_done( u_long sessionId )
{
    printf( "PLAY done.\n" );
#ifdef AUDIO
    close (wav_fd);
    wav_fd = 0;
    wav_session = 0;
#endif  /* AUDIO */
}

void
interface_error( const char *err )
{
    printf( err );
}

void
interface_alert_msg( const char *msg )
{
    printf( msg );
}
