/*
 * RTSP (Real Time Streaming Protocol) Reference Implementation
 * 
 *     Configuration File Handling
 *
 *  Copyright (C) 1996 Progressive Networks.
 * 
 *  This program is free software; you can only redistribute it and/or
 *  modify it under the terms of version 2 of the GNU General Public
 *  License as published by the Free Software Foundation.  A copy of this
 *  license may be found in the "LICENSE" directory of this distribution.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY OR REPRESENTATION AS TO OWNERSHIP OR
 *  NON-INFRINGEMENT; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 *  License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 *  Please submit patches, modified versions and derivative works to
 *  Progressive Networks at "rtsp-feedback@prognet.com" or write to
 *  Progressive Networks, 1111 Third Ave. Ste 2900, Seattle, WA 98101, USA.
 *  We will do our best to incorporate your enhancements into the core
 *  distribution.
 *
 *  For more information about this program, please visit the following
 *  URL:
 *     http://www.real.com/prognet/rt 
 *

 *
 * $Id: config.c,v 1.15 1997/05/30 04:54:03 robla Exp $
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#if defined (WIN32)
#include <io.h>
#else
#include <sys/types.h>
#include <sys/uio.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#endif

#include "machdefs.h"
#include "rtsp.h"
#include "config.h"
#include "util.h"

extern void terminate( int code );

#define MAXURLS 10

struct url_info *urls[MAXURLS];
static int n_urls = 0;
static u_short port = RTSP_DEFAULT_PORT;
char BasePath [MAX_BASE_PATH] = "";

u_short
get_config_port()
{
    return port;
}

void
init_config(int argc, char **argv)
{
   FILE *f;
   int i;

   if (argc < 2)
   {
      printf("No configuration file specified.  "
             "usage: rtsp-server <config file>\n");
      return;
   }

   for (i = 0; i < 10; i++)
       urls[i] = 0;

   f = fopen(argv[1], "r");
   if (!f)
   {
     printf( "Can't open configuration file '%s'.", argv[1] );
     terminate(-1);
   }

   i = 0;

   while (!feof(f))
   {
      char linebuf[256];

      if(!fgets(linebuf, sizeof(linebuf), f))
         break;
      if(linebuf[0] == '#' || linebuf[0] == '\n' || linebuf[0] == ' ')
         continue;

      if(strncasecmp(linebuf, "port:", 5) == 0)
      {
         /* skip 'port:' token */
         char *tok = strtok(linebuf, ":");
         tok = strtok(NULL, " \n");
         port = (u_short)atol(tok);
         continue;
      }

      if(strncasecmp(linebuf, "BasePath=", 9) == 0)
      {
         /* skip 'port:' token */
         char *tok = strtok(linebuf, "=");
         tok = strtok(NULL, " \n");
         strncpy( BasePath, tok, sizeof(BasePath) - 1 );
         continue;
      }
   }
}

struct url_info *
get_url_info(const char *urlpath)
{
    int i;
    for(i=0;i<n_urls;i++)
    {
	if(strcmp(urls[i]->urlpath, urlpath) == 0)
	    return urls[i];
    }
    return 0;
}
