/*
 * RTSP (Real Time Streaming Protocol) Reference Implementation
 * 
 *    Main server routine
 *
 *  Copyright (C) 1996 Progressive Networks.
 * 
 *  This program is free software; you can only redistribute it and/or
 *  modify it under the terms of version 2 of the GNU General Public
 *  License as published by the Free Software Foundation.  A copy of this
 *  license may be found in the "LICENSE" directory of this distribution.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY OR REPRESENTATION AS TO OWNERSHIP OR
 *  NON-INFRINGEMENT; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 *  License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 *  Please submit patches, modified versions and derivative works to
 *  Progressive Networks at "rtsp-feedback@prognet.com" or write to
 *  Progressive Networks, 1111 Third Ave. Ste 2900, Seattle, WA 98101, USA.
 *  We will do our best to incorporate your enhancements into the core
 *  distribution.
 *
 *  For more information about this program, please visit the following
 *  URL:
 *     http://www.real.com/prognet/rt 
 *

 *
 * $Id: main.c,v 1.17 1997/05/30 04:54:04 robla Exp $
 *
 */

#include <assert.h>
#include <stdio.h>
#ifdef INETD
#include <stdlib.h>
#include <syslog.h>
#endif

#include "machdefs.h"
#include "eventloop.h"
#include "socket.h"
#include "rtsp.h"
#include "config.h"
#include "server.h"

#ifdef WIN32
#include <windows.h>
#include <mmsystem.h>
#endif

extern void terminate(int errorcode);

#ifdef INETD
int printf (const char* fmt, ...)
{
   /* it would be nice to send all output to syslog file since inetd has redirected stdout. */
   return( 0 );
}
#endif

int
main (int argc, char **argv)
{
#ifndef INETD
   int      fd;
   int      lfd;
   int      size = 0;
   struct sockaddr s = { 0 };
#endif
   u_short  port;
#ifdef WIN32
   int      err;
   u_short  wVersionRequested; 
   WSADATA  wsaData;
   wVersionRequested = MAKEWORD(1, 1);
 
   /* make sure there is winsock.dll support */
   err = WSAStartup(wVersionRequested, &wsaData); 
   if ( err )
   {
      printf( "WSAStartup could not detect Windows socket support.\n" );
      terminate( err );
   }
#endif  

   init_server(argc, argv);
   port = get_config_port();

#ifndef INETD
   printf( "Listen on port %d.\n", port );
   lfd = tcp_listen(port);
   size = sizeof (s);
   fd = tcp_accept(lfd, &s, &size);
   printf( "TCP control channel established.\n" );
   main_fd = fd;
#ifndef WIN32
   main_fd_read = fd;
#endif
#else
   main_fd = 1;      /* inetd redirected std out */
   main_fd_read = 0; /* std in was also redirected */   

   inetd_init();
#endif
   eventloop_init();
   while (1)
      eventloop();
    return 0;
}
