/*
 * RTSP (Real Time Streaming Protocol) Reference Implementation
 * 
 *     Session Manager
 *
 *
 *  Copyright (C) 1996 Progressive Networks.
 * 
 *  This program is free software; you can only redistribute it and/or
 *  modify it under the terms of version 2 of the GNU General Public
 *  License as published by the Free Software Foundation.  A copy of this
 *  license may be found in the "LICENSE" directory of this distribution.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY OR REPRESENTATION AS TO OWNERSHIP OR
 *  NON-INFRINGEMENT; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 *  License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 *  Please submit patches, modified versions and derivative works to
 *  Progressive Networks at "rtsp-feedback@prognet.com" or write to
 *  Progressive Networks, 1111 Third Ave. Ste 2900, Seattle, WA 98101, USA.
 *  We will do our best to incorporate your enhancements into the core
 *  distribution.
 *
 *  For more information about this program, please visit the following
 *  URL:
 *     http://www.real.com/prognet/rt 
 *
 *
 * $Id: session.c,v 1.2 1997/04/01 17:31:14 robla Exp $
 *
 */

#include "session.h"

void
init_sessions(struct SESSION_STATE *session)
{
    int i;
    for(i=0;i<MAXSESSIONS;i++)
    {
	session[i].session_id = INVALID_SESSION;
	session[i].data = 0;
    }
}

void
add_session(u_long session_id, int state, struct SESSION_STATE *session)
{
    int i;
    for(i=0;i<MAXSESSIONS;i++)
    {
        if(session[i].session_id == INVALID_SESSION)
        {
            session[i].session_id = session_id;
            session[i].cur_state = state;
            session[i].data = 0;
	    return;
        }
    }
}

void
remove_session(u_long session_id, struct SESSION_STATE *session)
{
    int i;
    for(i=0;i<MAXSESSIONS;i++)
    {
        if(session[i].session_id == session_id)
	{
            session[i].session_id = INVALID_SESSION;
	    return;
	}
    }
}

int
get_active_sessions(struct SESSION_STATE *session)
{
    int i;
    int count = 0;
    for(i=0;i<MAXSESSIONS;i++)
    {
        if(session[i].session_id != INVALID_SESSION)
            count++;
    }
    return count;
}

struct SESSION_STATE *
find_session(u_long session_id, struct SESSION_STATE *session)
{
    int i;
    for(i=0;i<MAXSESSIONS;i++)
    {
        if(session[i].session_id == session_id)
	    return &session[i];
    }
    return 0;
}

void
set_session_url(u_long session_id, const char* url, struct SESSION_STATE *session)
{
    struct SESSION_STATE *s = find_session(session_id, session);
    if(s)
        strcpy(s->url, url);
}

const char *
get_session_url(u_long session_id, struct SESSION_STATE *session)
{
    struct SESSION_STATE *s = find_session(session_id, session);
    if(s)
        return s->url;
    return 0;
}


void
set_session_data(u_long session_id, void *data, struct SESSION_STATE *session)
{
    struct SESSION_STATE *s = find_session(session_id, session);
    if(s)
        s->data = data;
}

void *
get_session_data(u_long session_id, struct SESSION_STATE *session)
{
    struct SESSION_STATE *s = find_session(session_id, session);
    if(s)
        return s->data;
    return 0;
}

int
get_session_state(u_long session_id, struct SESSION_STATE *session)
{
    struct SESSION_STATE *s = find_session(session_id, session);
    if(s)
        return s->cur_state;
    return INVALID_SESSION;
}

void
set_session_state(u_long session_id, int state, struct SESSION_STATE *session)
{
    struct SESSION_STATE *s = find_session(session_id, session);
    if(s)
        s->cur_state = state;
}
