/*
 *  CAsyncNotify
 * 
 *	Notifies us of asynchronous events, this window is
 *	used to notify us that we can read fromt the network
 *	and when the sound device has finished playing a sound
 *	block
 *
 *  Copyright (C) 1996 Progressive Networks.
 * 
 *  This program is free software; you can only redistribute it and/or
 *  modify it under the terms of version 2 of the GNU General Public
 *  License as published by the Free Software Foundation.  A copy of this
 *  license may be found in the "LICENSE" directory of this distribution.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY OR REPRESENTATION AS TO OWNERSHIP OR
 *  NON-INFRINGEMENT; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 *  License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 *  Please submit patches, modified versions and derivative works to
 *  Progressive Networks at "rtsp-feedback@prognet.com" or write to
 *  Progressive Networks, 1111 Third Ave. Ste 2900, Seattle, WA 98101, USA.
 *  We will do our best to incorporate your enhancements into the core
 *  distribution.
 *
 *  For more information about this program, please visit the following
 *  URL:
 *     http://www.real.com/prognet/rt 
 */                
 
#include "stdafx.h"
#include <mmsystem.h>
 
#include "cwaveout.h"
#include "casync.h"  
 

CAsyncNotify::CAsyncNotify()
{
	m_hWnd = NULL;
	m_sessionClient = NULL;
	m_waveClient = NULL;
}

CAsyncNotify::~CAsyncNotify()
{
	if (m_hWnd && IsWindow( m_hWnd ))
	{
		DestroyWindow( m_hWnd );
	}
	
	if (m_bClassRegistered)
	{
		UnregisterClass( WND_CLASS, m_hInst );
	}
	
	m_hWnd = NULL;
	m_bClassRegistered = FALSE;
	m_waveClient = NULL;
	m_hInst = NULL;
}

BOOL CAsyncNotify::Create( CWnd *sessionClient, HINSTANCE hInst )
{
    WNDCLASS internalClass;

	m_hInst = hInst;
	
	//	First register our window class                                  
 	internalClass.style 		= 0;
	internalClass.lpfnWndProc 	= CAsyncNotify::AsyncNotifyProc;
	internalClass.cbClsExtra    = 0;
	internalClass.cbWndExtra    = sizeof( this );
  	internalClass.hInstance     = m_hInst; // Use app's instance
	internalClass.hIcon         = 0;
	internalClass.hCursor       = 0;
	internalClass.hbrBackground = 0;
	internalClass.lpszMenuName  = NULL;
	internalClass.lpszClassName = WND_CLASS;
	if (!(m_bClassRegistered = RegisterClass( &internalClass )))
	{
		return( FALSE );
	}

	//	Now create an instance of the window	
	m_hWnd = CreateWindow( "RealAudioInternal", 
	 "RealAudio Internal Messages", WS_OVERLAPPED, 
     0, 0, 0, 0, NULL, NULL, hInst, NULL ); 
     
    if (!m_hWnd)
    {
    	return( FALSE );
    }
    
    //	Set our this pointer, so our WndProc can find us again
    SetWindowLong( m_hWnd, 0, (long) this );
    
    m_sessionClient = sessionClient;
    return( TRUE );
}

void CAsyncNotify::SetWaveClient( CWaveOutput *waveClient )
{
	m_waveClient = waveClient;
}

LRESULT CALLBACK
CAsyncNotify::AsyncNotifyProc( HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
	CAsyncNotify	*THIS;
	
	//	Note:	This MUST be a static function
	//	However we make it appear to be a member function
	//	By imbedding a this pointer in the window object
	THIS = (CAsyncNotify far *)(LPHANDLE)GetWindowLong( hwnd, 0 );

	switch (msg)
	{
	//	Keep MM_WOM_DONE && PWM_ASYNC_SELECT at the top since they're probably 
	//	our most popular messages by far
	case MM_WOM_DONE:              
		return( THIS->OnWaveOutDone( (HWAVEOUT)wParam, (CWaveHeader FAR *)(LPVOID)lParam ) );

	case PWM_ASYNC_SELECT:
		return( THIS->OnAsyncSelect( wParam, lParam ) );

	case PWM_POSTED_IDLE:
		THIS->OnPostedIdle();
		return( 1L );

	case PWM_POSTED_ERROR:
		THIS->OnPostedError( (int)wParam, (LPVOID)lParam );
		return( 1L );
			
	default:
		break;
	}
	return( DefWindowProc( hwnd, msg, wParam, lParam ) );
}

BOOL CAsyncNotify::OnAsyncSelect( WPARAM wParam, LPARAM lParam )
{
	return( TRUE );
}

BOOL CAsyncNotify::OnWaveOutDone( HWAVEOUT hwave, CWaveHeader FAR *pwhDone )
{
	m_waveClient->DonePlaying( hwave, pwhDone ); 
	 
	return (0);
}                  

void CAsyncNotify::PostIdleMessage( void )
{
	MSG		msg;

	if (m_hWnd && this && IsWindow( m_hWnd ))
	{
		//	Only posts the message if there are NO other idle messages in the que.
		if (PeekMessage( &msg, m_hWnd, PWM_POSTED_IDLE, PWM_POSTED_IDLE, PM_NOREMOVE | PM_NOYIELD ))
		{
			return;
		}
		PostMessage( m_hWnd, PWM_POSTED_IDLE, 0, 0L );
	}
}

//	Posts an error condition to the Async Window which can then dispatch the
//	error context to a handler
void CAsyncNotify::PostAsyncError( int theError, VOID FAR *lpErrorContext )
{
	if (m_hWnd && this && IsWindow( m_hWnd ))
	{
		PostMessage( m_hWnd, PWM_POSTED_ERROR, (WPARAM)theError, (LPARAM)lpErrorContext );
	}
}

void CAsyncNotify::OnPostedIdle()
{
	return;
}

void CAsyncNotify::OnPostedError( int theError, LPVOID lpErrorContext )
{
}
