/*
 *  CWaveOut
 * 
 *  Copyright (C) 1996 Progressive Networks.
 * 
 *  This program is free software; you can only redistribute it and/or
 *  modify it under the terms of version 2 of the GNU General Public
 *  License as published by the Free Software Foundation.  A copy of this
 *  license may be found in the "LICENSE" directory of this distribution.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY OR REPRESENTATION AS TO OWNERSHIP OR
 *  NON-INFRINGEMENT; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 *  License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 *  Please submit patches, modified versions and derivative works to
 *  Progressive Networks at "rtsp-feedback@prognet.com" or write to
 *  Progressive Networks, 1111 Third Ave. Ste 2900, Seattle, WA 98101, USA.
 *  We will do our best to incorporate your enhancements into the core
 *  distribution.
 *
 *  For more information about this program, please visit the following
 *  URL:
 *     http://www.real.com/prognet/rt 
 *

 */                
 
#include <assert.h>
#include "stdafx.h"
#include <stdlib.h>
#include <mmsystem.h>

#include "cwaveout.h"

#define WAIT_AT_THE_END 5

LRESULT CALLBACK 
CWaveOutputNotificationProc( HWND hwnd, UINT msg, WPARAM wParam,
 LPARAM lParam );

const unsigned short CWaveHeader::WAVE_BLOCKSIZE = 16384;
const short CWaveOutput::NUM_WAVEHDRS = 30;

BOOL OSYield( void ) 
{
	MSG		msg;
	/*
	**	We need to call PeekMessage for all
	**	messages or else Windows doesn't yield properly.
	*/
	while (PeekMessage(&msg, NULL,0,0,PM_REMOVE)) 
	{
       	if (msg.message == WM_QUIT) 
       	{
       		/* Application is being exited */
       		return FALSE;
        }
       	TranslateMessage(&msg);
       	DispatchMessage(&msg);
    }
    return TRUE;
}

CWaveHeader::CWaveHeader() 
{
	memset( &wh, 0, sizeof(wh) );
	fAvail = FALSE;
	filler = 0;
 	timeEnd = 0;   
}	

CWaveOutput::CWaveOutput( CAsyncNotify *pAsNotif ) 
{
	AsNotif = pAsNotif;
	mcbPlaying = 0L;
	//	What about wf? It's zero inited by it's constructor
	pWaveHdrs = NULL;
	pWaveBuffers = NULL;
	mhwave = NULL;
	mcbAvailable = 0L;
	mBlocksQueued = 0;
	aborting = 0;
	resetting = 0;
	opened = 0;
	mpbAddUnpacked=NULL;
	pos=0;                                 
    FlowBuffer = new char[CWaveHeader::WAVE_BLOCKSIZE];		
}                

CWaveOutput::~CWaveOutput(void) 
{
	Abort();
	if (pWaveHdrs)
	{
		delete [] pWaveHdrs;
	}
	if (pWaveBuffers)
	{
		delete [] pWaveBuffers;
	}
	if (mpbAddUnpacked) {
		delete [] mpbAddUnpacked;
	}
	if (FlowBuffer)
	{
		delete[] FlowBuffer;
	}                
}

void CWaveOutput::DonePlaying( HWAVEOUT hWave, CWaveHeader FAR *pwhDone ) 
{
	unsigned short cbPlayed;
	
	//	Some error checks
	assert( hWave );
	assert( this );
//	DEBUGOUTSTR( "DonePlaying()\r\n" );

	if (!hWave)
	{  return;
	}
	pwhDone->fAvail = TRUE;
	waveOutUnprepareHeader( hWave, &pwhDone->wh, sizeof( WAVEHDR ) );
	
	if (!this)
	{
		// DEBUGOUTSTR( "this is NULL\r\n" );
		return;
	}
	cbPlayed = WORD( pwhDone->wh.dwBufferLength );
	 
	mcbPlaying -= cbPlayed;
	
	// Subtle point: entire blocks get used and returned.
	// Note that this function only works for single blocks
	// or less at a time.
	mcbAvailable += CWaveHeader::WAVE_BLOCKSIZE;
	// mcbAvailable += cbPlayed;
	--mBlocksQueued;
	
	TimeEndNotified = pwhDone->timeEnd;
}

//	Opens the wave device, returns type RaErr to indicate success or failure.
unsigned short CWaveOutput::Open(const char * pData, unsigned short cbData) 
{
    if(IsOpen())
      return 0;

   SetAudioFormat();	
	/*
	**	Open the wave driver.
	**	We try w/ multiple formats if necessary to get the right format 
	**	for the driver.
	*/               
	timeEnd=0;
	TimeEndNotified=0;
	
	if (!TryWaveOutOpen())
	{
		return 0;
	}

	//	Ok, the assumption here is that we've successfully opened the wave
	//	device, now we need to set a notifier for it.

	AsNotif->SetWaveClient(this);
	
	/*
	**	Allocate memory for wave block headers and for wave data.
	*/
	if (!pWaveHdrs && !(pWaveHdrs = new CWaveHeader [NUM_WAVEHDRS]))
	{
		goto FailOut;
	}
	
	if (!pWaveBuffers  && !(pWaveBuffers = (unsigned char *)( new BYTE [NUM_WAVEHDRS * CWaveHeader::WAVE_BLOCKSIZE] )))
	{
		goto FailOut;
	}
								
	/*
	**	Mark all wave block headers as available, and point them
	**	at their respective wave data blocks.
	*/
	short	iwh;
	
	for (iwh = 0; iwh < NUM_WAVEHDRS; iwh++) 
	{
		CWaveHeader *pwh = &pWaveHdrs[iwh];
#if !defined(WIN32) && !defined(_WIN32)  		
		pwh->wh.lpData = ( char *) (&pWaveBuffers[iwh * CWaveHeader::WAVE_BLOCKSIZE]);
#else
	    pwh->wh.lpData = ( char *) (&pWaveBuffers[iwh * CWaveHeader::WAVE_BLOCKSIZE]);
#endif
		pwh->wh.dwUser = DWORD (this);
		pwh->fAvail = TRUE;
	}
	
	/*
	**	Indicate that no audio blocks are currently playing.
	*/
	mcbPlaying = 0;
	mBlocksQueued = 0;
	mcbAvailable = NUM_WAVEHDRS * CWaveHeader::WAVE_BLOCKSIZE;

	/*
	**	Set some success flags and get out.
	*/	
	aborting = FALSE;
	resetting = FALSE;
	opened = TRUE;
//	return offset;
	return 1;   // OK

FailOut:
	if (pWaveHdrs)
	{
		delete [] pWaveHdrs;
	}
	if (pWaveBuffers)
	{
		delete [] pWaveBuffers;
	}
	return 0;
	
}

//	Try to open the wave device, we are willing to try
//	multiple waveformat types to get this right.  We can
//	also fail if the wave device is allocated or it returns
//	some other source of error.
BOOL CWaveOutput::TryWaveOutOpen( void )
{
	UINT			wRet;	
	LPWAVEFORMATEX	wavePtr;

	
	wavePtr = wf.GetWaveFormat();
	//	Try... to open the wave device with an "adjustable"
	//	wave header.    
	wRet = waveOutOpen(&mhwave, WAVE_MAPPER, wavePtr,
	                   AsyncNotifyWnd(), 0L, CALLBACK_WINDOW );


//debug
//if (wRet)
//{
//   char b [500];
//   waveOutGetErrorText(wRet, b, sizeof(b));
//	AfxMessageBox( b );
//   return FALSE;
//}
//debug

	//	Okay, translate any error returns and get out if there was an error
	switch (wRet)
	{
	//	No error get on with it
	case MMSYSERR_NOERROR:
		return TRUE;

	case MMSYSERR_ALLOCATED:
	case MMSYSERR_NOMEM:
	case MMSYSERR_BADDEVICEID:
	case WAVERR_BADFORMAT:
	//	A syncronous driver can't support async writes
	case WAVERR_SYNC:
	//	Shouldn't have one of these...	but what the hell
	default:
		return FALSE;
	}
}

void CWaveOutput::Abort(void) 
{
	// DEBUGOUTSTR( "aborting being set to TRUE\r\n" );
    aborting = TRUE;
	Reset();
	// Close the wave driver
	if (mhwave != NULL) 
	{
		int		i;
		
		for (i = 0; i < 5 && (WAVERR_STILLPLAYING == waveOutClose( mhwave )); ++i)
		{
			// OutputDebugString( "**** WaveOutClose failed first time\r\n" );
			waveOutReset( mhwave );
		}
		mhwave = NULL;
	}                     
	AsNotif->SetWaveClient(NULL);
	opened = FALSE;
}

void CWaveOutput::Reset(void) 
{
	// OutputDebugString( "CWaveOutput::Reset()\r\n" );
	resetting = TRUE;
	if (!opened) 
	{
		return;
	}
	
	if (mhwave != NULL) 
	{
		waveOutReset (mhwave);
	}

	// Make sure we handle all the wave-done messages,
	// but establish a time limit to cope with the 
	// unpredictable nature of Windows. Wait for up
	// to 50 milliseconds.
	DWORD start = GetTickCount();
	while (IsActive ()) 
	{
		if (!OSYield()) 
		{
			break;
		}
		if (GetTickCount() - start >= 50) 
		{
			break;
		}
	}
	// OutputDebugString( "Ok, Stopped waiting for wave-done messages\r\n" );
	
	// Force the counters back to full in case messages were lost
	mcbPlaying = 0;
	mcbAvailable = NUM_WAVEHDRS * CWaveHeader::WAVE_BLOCKSIZE;
	// Clean up any blocks still unaccounted for
	for (int i=0; (i < NUM_WAVEHDRS); i++) 
	{
		CWaveHeader *pwhDone = &pWaveHdrs[i];
		
		if (!pwhDone->fAvail) 
		{
			waveOutUnprepareHeader( mhwave, LPWAVEHDR (&(pwhDone->wh)),
			 sizeof( WAVEHDR ) );
			pwhDone->fAvail = TRUE;
		}
	}
	resetting = FALSE;
}

BOOL  CWaveOutput::Play(const char * lpbData, unsigned short cbData)
{
	short	iwh;
	int		old_pos; 

	if(!IsOpen())
	{
//		unsigned short offset = Open(lpbData, cbData);
//		if(!offset)
//	        return FALSE;
//		cbData -= offset;
//		lpbData += offset;
		if (!Open(lpbData, cbData) )
         return FALSE;
	}

	if(!SpaceAvailable(cbData))                               
	    return FALSE; 

	if(!FlowBuffer)
	  return FALSE;
	old_pos = pos;       
	pos += (pos + cbData <= CWaveHeader::WAVE_BLOCKSIZE ? cbData : CWaveHeader::WAVE_BLOCKSIZE - pos);
	 
	if(lpbData) 
	{
	     memcpy( FlowBuffer + old_pos, lpbData, pos - old_pos );
	}

		   // if !lpbData - we must flush current buffer                                           
	if (lpbData && pos != CWaveHeader::WAVE_BLOCKSIZE)
	{
		return TRUE;
	}
	
	if (!mhwave)
	{
		return TRUE;
	}   

	//	Look for an available audio buffer
	for (iwh = 0; iwh < NUM_WAVEHDRS; iwh++) 
	{
		CWaveHeader *pwh = &pWaveHdrs[iwh];
			
		if (!pwh->fAvail) 
		{
			/*
			**	This block is busy 
			*/
			continue;
		}
		
			/*
		**	Got an available block. Copy the audio data into it.
		*/
		memcpy (pwh->wh.lpData, FlowBuffer, pos); 
		pwh->wh.dwBufferLength = pos;

		if(lpbData)
		  memcpy(FlowBuffer, lpbData + pos - old_pos, cbData - (pos - old_pos));
		pos = cbData - (pos - old_pos);
	     
		UINT	wRet;
		
		if ((wRet = waveOutPrepareHeader( mhwave, LPWAVEHDR (&pwh->wh),	sizeof (WAVEHDR))) != 0) 
		{
			return FALSE;
		}
		if ((wRet = waveOutWrite( mhwave, LPWAVEHDR( &pwh->wh ), sizeof (WAVEHDR))) != 0) 
		{
			return FALSE;
		}

		/*
		**	The block is currently playing.
		*/
		pwh->fAvail = FALSE;
		pwh->timeEnd = timeEnd;        
		/* */ 
		timeEnd++;
		StartedPlaying( cbData );
		++mBlocksQueued;
		  
		// OutputDebugString( "Succeeding out of CWaveOuput::Play()\r\n" );
		return TRUE;
	}
	// No blocks open
	return FALSE;
}
				
void CWaveOutput::Pause(void) 
{
	if (mhwave) 
	{
		waveOutPause( mhwave );
    }
}

void CWaveOutput::Resume(void) 
{
	if (mhwave) 
	{
		waveOutRestart( mhwave );
	}
}

UINT CWaveOutput::AsyncNotifyWnd()
{
	return (UINT)(AsNotif->GetHwnd());
}
 
BOOL CWaveOutput::FinishPlaying()
{ static int counter=0;
  static RaAudioTime TimeCount=TimeEndNotified;		
  if(TimeCount==TimeEndNotified)
    counter++;
  else
  { counter=0;
    TimeCount=TimeEndNotified;		
  }                                 
  if(!TimeEndNotified || !timeEnd || counter>=WAIT_AT_THE_END && TimeEndNotified==timeEnd-2)
    return TRUE;
  return (TimeEndNotified>=timeEnd-1);
}

void CWaveOutput::Flush( BOOL MustPlay )
{ 
	if (MustPlay && pos)
	{
		Play( NULL, 0 ) ;
	}
  	else
  	{
		pos=0;
	}
}
