/*
 *	cwaveout.h
 *
 *  Copyright (C) 1996 Progressive Networks.
 * 
 *  This program is free software; you can only redistribute it and/or
 *  modify it under the terms of version 2 of the GNU General Public
 *  License as published by the Free Software Foundation.  A copy of this
 *  license may be found in the "LICENSE" directory of this distribution.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY OR REPRESENTATION AS TO OWNERSHIP OR
 *  NON-INFRINGEMENT; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 *  License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 *  Please submit patches, modified versions and derivative works to
 *  Progressive Networks at "rtsp-feedback@prognet.com" or write to
 *  Progressive Networks, 1111 Third Ave. Ste 2900, Seattle, WA 98101, USA.
 *  We will do our best to incorporate your enhancements into the core
 *  distribution.
 *
 *  For more information about this program, please visit the following
 *  URL:
 *     http://www.real.com/prognet/rt 
 *

 *
 * $Id: CWaveOut.h,v 1.6 1997/04/01 17:31:27 robla Exp $
 */

#ifndef CWAVEOUT_H
#define CWAVEOUT_H 1

#include "cwavefmt.h"
#include "casync.h"

typedef	unsigned long  RaAudioTime;

class CWaveOutput;
class CAsyncNotify;

/*--------------------------------------------------------------------------
|	MYWAVEHDR structure
|
|		Superset of WAVEHDR for wave audio playback.
--------------------------------------------------------------------------*/
class CWaveHeader 
{
	friend class CWaveOutput;
	friend class CAsyncNotify;
			
protected:
	WAVEHDR			wh;
	unsigned char	fAvail;
	unsigned char	filler;
	CWaveHeader();

public:
	static	const	unsigned short	WAVE_BLOCKSIZE;
	RaAudioTime 	timeEnd;
};

/*--------------------------------------------------------------------------
|	CWaveOutput
--------------------------------------------------------------------------*/
class CWaveOutput 
{
public:
	CWaveOutput		( CAsyncNotify *pAsNotif );
	~CWaveOutput	( void );

private:
	static const short NUM_WAVEHDRS;
	CAsyncNotify	* AsNotif;
	unsigned short	mcbPlaying;
	CWaveFormat		wf;
	CWaveHeader		*pWaveHdrs;
	unsigned char   *pWaveBuffers;		
	HWAVEOUT		mhwave;
	unsigned int	mcbAvailable;
	int				mBlocksQueued;
   	int				aborting;
	int				resetting;
	int				opened;
	char *       	mpbAddUnpacked;			// additional buffer for uosampling
	RaAudioTime		TimeEndNotified;
    RaAudioTime		timeEnd;	
    
public:	
 								
	inline	BOOL	IsOpen()
					{ 
						return( opened ); 
					}
	BOOL    FinishPlaying();
							
	inline	int		IsActive( void )
					{ 
						return( mBlocksQueued );
					}
				 								

	void			Abort( void );
	void			Reset( void );
	BOOL    		Play( const char * lpbData, unsigned short cbData );
	void			Flush( BOOL MustPlay);
				
	void			Pause( void );
	void			Resume( void );
	                                            
	inline	int		SpaceAvailable( unsigned short desired )
					{
						return( mBlocksQueued < NUM_WAVEHDRS );
					}
								
	// Subtle point: entire blocks get used and returned.
	// Note that this function only works for single blocks
	// or less at a time.
	void			DonePlaying( HWAVEOUT hWave, CWaveHeader FAR *pwhDone );

private: 

	unsigned short Open(const char * pData, unsigned short cbData);

	unsigned short ParseWavData(const char * pData, unsigned short cbData);

	BOOL    	   TryWaveOutOpen( void );

	void  SetAudioFormat( void )
	{  
		 wf.SetAudioFormat();
	};

	// Subtle point: entire blocks get used and returned.
	// Note that this function only works for single blocks
	// or less at a time.
	inline	void	StartedPlaying( unsigned short cbPlayed )
					{
						mcbPlaying += cbPlayed;
						mcbAvailable -= CWaveHeader::WAVE_BLOCKSIZE;
					}   
	char 			*FlowBuffer;     						
	int  			pos;
								                      
	//	These guys are called by the CWaveOutput() objects and they
	//	call up the object hierarchy to the CAsyncNotify object
	UINT        	AsyncNotifyWnd();
};

#endif /* CWAVEOUT_H */
	
