/*
 * RTSP (Real Time Streaming Protocol) Reference Implementation
 *
 *     Player app
 *
 *
 *  Copyright (C) 1996 Progressive Networks.
 * 
 *  This program is free software; you can only redistribute it and/or
 *  modify it under the terms of version 2 of the GNU General Public
 *  License as published by the Free Software Foundation.  A copy of this
 *  license may be found in the "LICENSE" directory of this distribution.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY OR REPRESENTATION AS TO OWNERSHIP OR
 *  NON-INFRINGEMENT; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 *  License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 *  Please submit patches, modified versions and derivative works to
 *  Progressive Networks at "rtsp-feedback@prognet.com" or write to
 *  Progressive Networks, 1111 Third Ave. Ste 2900, Seattle, WA 98101, USA.
 *  We will do our best to incorporate your enhancements into the core
 *  distribution.
 *
 *  For more information about this program, please visit the following
 *  URL:
 *     http://www.real.com/prognet/rt 
 *

 *
 * $Id: RefPlayer.cpp,v 1.13 1997/06/09 22:30:33 robla Exp $
 *
 */

#include "stdafx.h"
#include "socket.h"
#include "RefPlayer.h"
#include "RefPlayerDlg.h"
#include "cwaveout.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern int  TIMER_ID;

CRefPlayerApp::CRefPlayerApp()
{
	asyncNotifier = NULL;
	waveOut = NULL;
}

CRefPlayerApp theApp;

extern "C"
{
    void terminate(int code);
}

void terminate(int code)
{
   theApp.m_pMainWnd->KillTimer( TIMER_ID );      // kill the event timer
   theApp.m_pMainWnd->SendMessage(WM_DESTROY);
}

BOOL Play(const char * pBuffer, UINT cbLength)
{
	return theApp.Play(pBuffer, cbLength);
}

void Flush(BOOL MustPlay)
{
	theApp.Flush(MustPlay);
}

BOOL CRefPlayerApp::Play(const char * pBuffer, UINT cbLength)
{
    return waveOut->Play(pBuffer, cbLength);
}

void CRefPlayerApp::Flush(BOOL MustPlay)
{
    waveOut->Flush(MustPlay);
}

BOOL CRefPlayerApp::InitInstance()
{
    if (!AfxSocketInit())
    {
	AfxMessageBox(IDP_SOCKETS_INIT_FAILED);
	return FALSE;
    }

#ifdef _AFXDLL
    Enable3dControls();
#else
    Enable3dControlsStatic();
#endif

    CCommandLineInfo cmdInfo;
    ParseCommandLine(cmdInfo);
    if(cmdInfo.m_nShellCommand == CCommandLineInfo::FileOpen)
	getURL(cmdInfo.m_strFileName, m_url);

    CRefPlayerDlg dlg(m_url);
    m_pMainWnd = &dlg;

	asyncNotifier = new CAsyncNotify;                         
	asyncNotifier->Create(m_pMainWnd, AfxGetInstanceHandle());
	waveOut=new CWaveOutput(asyncNotifier);
	
    dlg.DoModal();

	if(waveOut->IsOpen())  
        waveOut->Abort();     
	
	delete waveOut;
	delete asyncNotifier;
	
    return FALSE;
}


BOOL CRefPlayerApp::getURL(const char *fileName, CString &url) 
{
    FILE *fp = fopen(fileName, "r");
    if(fp)
    {
	char linebuf[256];
	if(fgets(linebuf, sizeof(linebuf), fp))
	{
	    m_url = strtok(linebuf, " \n\t");
	    fclose(fp);
	    return TRUE;
	}
    }
    fclose(fp);
    return FALSE;
}

