/*
 *  RefPlayerDlg
 * 
 *  Copyright (C) 1996 Progressive Networks.
 * 
 *  This program is free software; you can only redistribute it and/or
 *  modify it under the terms of version 2 of the GNU General Public
 *  License as published by the Free Software Foundation.  A copy of this
 *  license may be found in the "LICENSE" directory of this distribution.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY OR REPRESENTATION AS TO OWNERSHIP OR
 *  NON-INFRINGEMENT; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 *  License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 *  Please submit patches, modified versions and derivative works to
 *  Progressive Networks at "rtsp-feedback@prognet.com" or write to
 *  Progressive Networks, 1111 Third Ave. Ste 2900, Seattle, WA 98101, USA.
 *  We will do our best to incorporate your enhancements into the core
 *  distribution.
 *
 *  For more information about this program, please visit the following
 *  URL:
 *     http://www.real.com/prognet/rt 
 *
 */


#include "stdafx.h"
#include "socket.h"
#include "RefPlayer.h"
#include "StreamDlg.h"
#include "interface.h"
#include "util.h"
#include "player.h"
#include "session.h"

extern "C" struct SESSION_STATE  p_state;

int TIMER_ID = 100;
const int TIMEOUT = 10;	/* milliseconds */

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

class CAboutDlg : public CDialog
{
public:
    CAboutDlg() {};

    enum { IDD = IDD_ABOUTBOX };
};

CRefPlayerDlg::CRefPlayerDlg(const char *url, CWnd* pParent /*=NULL*/)
    : CDialog(CRefPlayerDlg::IDD, pParent),
    m_url(url)
{
    m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

CRefPlayerDlg::~CRefPlayerDlg()
{
}

BEGIN_MESSAGE_MAP(CRefPlayerDlg, CDialog)
    ON_WM_SYSCOMMAND()
    ON_WM_PAINT()
    ON_WM_QUERYDRAGICON()
    ON_WM_TIMER()
    ON_WM_DESTROY()
    ON_BN_CLICKED(IDC_GO_BN, OnGoBn)
    ON_BN_CLICKED(IDC_HELP_BTN, OnHelpBtn)
END_MESSAGE_MAP()


BOOL CRefPlayerDlg::OnInitDialog()
{
    CDialog::OnInitDialog();

    ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
    ASSERT(IDM_ABOUTBOX < 0xF000);

    CMenu* pSysMenu = GetSystemMenu(FALSE);
    CString strAboutMenu;
    strAboutMenu.LoadString(IDS_ABOUTBOX);
    CString strHelpMenu;
    strHelpMenu.LoadString(IDS_HELP);
    if (!strHelpMenu.IsEmpty())
    {
	    pSysMenu->AppendMenu(MF_SEPARATOR);
	    pSysMenu->AppendMenu(MF_STRING, IDC_HELP_BTN, strHelpMenu);
    }
    if (!strAboutMenu.IsEmpty())
    {
	    pSysMenu->AppendMenu(MF_SEPARATOR);
	    pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strAboutMenu);
    }

    SetIcon(m_hIcon, TRUE);	// Set big icon
    SetIcon(m_hIcon, FALSE);	// Set small icon

    m_cmdED.SubclassDlgItem(IDC_URL_ED, this);
    m_textED.SubclassDlgItem(IDC_TEXT_ED, this);

    // initialize eventloop
    eventloop_init();
    init_player();
    interface_init(this);
    initSessionDlg();

    SetWindowText("RTSP Reference Player - unconnected");

    // check for url set
   if(m_url.GetLength() > 0)
   {
      CString cmd = "o " + m_url;
      m_cmdED.SetWindowText(cmd);
      OnGoBn();
   }
   else
   {
      m_cmdED.SetWindowText("o rtsp://");
   }

	m_timer = SetTimer(TIMER_ID, TIMEOUT, 0);
   return TRUE;
}

void CRefPlayerDlg::KillEventTimer( void )
{
	KillTimer( TIMER_ID );
}

void CRefPlayerDlg::OnDestroy()
{
   PostQuitMessage( 0 );      // exit code
}

void CRefPlayerDlg::OnSysCommand(UINT nID, LPARAM lParam)
{
    if ((nID & 0xFFF0) == IDM_ABOUTBOX)
    {
	CAboutDlg dlgAbout;
	dlgAbout.DoModal();
    }
    else
    {
	CDialog::OnSysCommand(nID, lParam);
    }
}

void CRefPlayerDlg::OnPaint() 
{
    if (IsIconic())
    {
	CPaintDC dc(this); // device context for painting

	SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

	// Center icon in client rectangle
	int cxIcon = GetSystemMetrics(SM_CXICON);
	int cyIcon = GetSystemMetrics(SM_CYICON);
	CRect rect;
	GetClientRect(&rect);
	int x = (rect.Width() - cxIcon + 1) / 2;
	int y = (rect.Height() - cyIcon + 1) / 2;

	// Draw the icon
	dc.DrawIcon(x, y, m_hIcon);
    }
    else
    {
	CDialog::OnPaint();
    }
}

HCURSOR CRefPlayerDlg::OnQueryDragIcon()
{
    return (HCURSOR) m_hIcon;
}

void CRefPlayerDlg::OnTimer(UINT id)
{
	for(int i = 0; i < 5; i++)
        eventloop();
}

void CRefPlayerDlg::OnHelpBtn()
{
static const char sHelpMsg [] =
           "Player commands:\n\n"
           "    o)pen     establishes RTSP control channel with the specified server.\n"
           "        format:     o [rtsp://]server[:port]\n"
           "        examples:   o bigserver\n"
           "                    o rtsp://mediaserver:9095\n"
           "    g)et      gets the specified file's description and\n"
           "              sets up server and player to play.\n"
           "        format:     g [[/]rel-path/]filename\n"
           "        examples:   g audio.wav\n"
           "                    g /sounds_dir/drip.wav\n"
           "                    g ascii.txt\n"
           "    p)lay     retrieves and plays/displays streamed data. A\n"
           "              range may be specified in smpte format.\n"
           "        format:     p [[smpte=]00:00:10-[00:01:30]]\n"
           "        examples:   p\n"
           "                    p 00:00:20-\n"
           "                    p smpte=00:00:20-00:01:30\n"
           "    c)lose          c\n"
           "    d)ebug toggle   d\n"
           "    h)elp           h\n"
           "    q)uit           q\n"
           "                    x\n";

   AfxMessageBox( sHelpMsg, MB_ICONQUESTION | MB_OK, 0 );
}

void CRefPlayerDlg::OnGoBn()
{
   CString  cmd;
   char     c;
   int      i;

   m_cmdED.GetWindowText(cmd);
   if ( cmd.IsEmpty() )
      goto BlankLine;
   cmd.TrimLeft();
   if ( cmd.IsEmpty() )
      goto BlankLine;
   c = tolower(cmd.GetAt(0));
   if (c == 'o')
   {
      // parse the command to get the server name
      // I know this is ugly
      char tokbuf[1024];
      strcpy(tokbuf, (LPCTSTR)cmd);
      char *tok = strtok(tokbuf, " \t\n");	// get 'open'
      if(tok)
      {
         char server[256];
         char filename[256];
         u_short port;

         tok = strtok(NULL, " \t\n");	// get url
         if(parse_url(tok, server, &port, filename))
         {
            m_serverName = server;
            m_fileName = filename;
            m_port = port;
         }
      }
   }
   else if (c == 'g')
   {  
      /*
       * on `get' grab the path/filename if present and set the filename.
       * 'open' doesn't require a filename so it may not be present until 'get'.
       * m_fileName is used in ::streamOutput() to determine if the content is
       * a wave file.
       */
      cmd.TrimRight();
      i = cmd.ReverseFind( ' ' );
      if ( i != -1 )
      {
         m_fileName = cmd.Mid( i );
         m_fileName.TrimLeft();
      }

   }
   handle_command(LPCTSTR(cmd));
   return;

BlankLine:
   AfxMessageBox("Enter a command line.");
   m_cmdED.SetWindowText("o ");
   return;
}

void CRefPlayerDlg::newSession(u_long sessionId)
{
    m_cmdED.SetWindowText("p");
    CStreamDlg *dlg = new CStreamDlg(sessionId, this);
    dlg->create();
    if(!addSessionDlg(sessionId, dlg))
	AfxMessageBox("Unable to add session");
}

void CRefPlayerDlg::renewPlay( u_long sessionId )
{
   CStreamDlg *dlg = (CStreamDlg *)getSessionDlg( sessionId );
   if ( dlg )
      dlg->RepositionSeekSlider( 0 );
   m_cmdED.SetWindowText("p");
}

void CRefPlayerDlg::startPlay( u_long sessionId )
{
   CStreamDlg *dlg = (CStreamDlg *)getSessionDlg( sessionId );
   if ( dlg )
      dlg->DisablePlayBtn();
}

void CRefPlayerDlg::closeSession(u_long sessionId)
{
   CStreamDlg *dlg = (CStreamDlg *)getSessionDlg(sessionId);
   if(dlg)
   {
      dlg->closeStream();
   }
   removeSessionDlg(sessionId);
}

void CRefPlayerDlg::hello()
{
    SetWindowText("RTSP Reference Player - connected to " + m_serverName);
    m_cmdED.SetWindowText("");
    if(m_fileName.GetLength() > 0)
    {
        char cmdbuf[20];
        sprintf(cmdbuf, "g %s", m_fileName);
        handle_command(cmdbuf);
    }
}

void CRefPlayerDlg::displayError(const char *err)
{
    AfxMessageBox(err);
}

void CRefPlayerDlg::displayAlert(const char *msg)
{
    AfxMessageBox( msg );
}

void CRefPlayerDlg::streamOutput(u_long sessionId, u_short len, const char *buf)
{
	if(!(m_fileName.Right(4)).CompareNoCase(".wav"))
	{
		if(Play(buf, len))
		    return;
	}

/*  no text support for now.
    CString str;
    for(int i=0;i<len;i++)
    {
	if(buf[i] == '\n')
	    str += "\r\n";
	else
	    str += buf[i];
    }
    CStreamDlg *dlg = (CStreamDlg *)getSessionDlg(sessionId);
    if(dlg)
	{
	    dlg->writeText(str);
	}
*/
}

void CRefPlayerDlg::streamDone( u_long sessionId )
{
   CStreamDlg *dlg = (CStreamDlg *)getSessionDlg(sessionId);
   Flush(1);
   if( dlg )
   {
      dlg->EnablePlayBtn();
      dlg->RepositionSeekSlider( 0 );
   }
}

void CRefPlayerDlg::resumeStream(u_long sessionId)
{
    char cmdbuf[20];
    strcpy( cmdbuf, "p" );    // play will resume.  Let server manage position.
    handle_command(cmdbuf);
}

void CRefPlayerDlg::startStream(u_long sessionId)
{
    char cmdbuf[20];
    strcpy( cmdbuf, "p" );
    handle_command(cmdbuf);
}

void CRefPlayerDlg::pauseStream(u_long sessionId)
{
    char cmdbuf[20];
    strcpy( cmdbuf, "z" );
    handle_command(cmdbuf);
}

void CRefPlayerDlg::setStreamSeek(u_long sessionId, int pos, int range)
{
   char  cmdbuf[30];
   int   hr, min, sec;
   long  seconds;
   float flt;
   long  max_sec;       // duration of clip in seconds
                    
   max_sec = p_state.settings.duration / 1000;
   // generate an smpte= time code.
   flt = ((float) max_sec) * pos / range;
   seconds = (long) flt;
   if ( !seconds && (seconds < max_sec) && (flt > 0.5) )
     seconds++;
   hr = (int) (seconds / 3600);
   seconds -= hr * 3600;
   min = (int) (seconds / 60);
   seconds -= min * 60;
   sec = (int) seconds;

   sprintf( cmdbuf, "p smpte=%2.2d:%2.2d:%2.2d-", hr, min, sec );
   handle_command(cmdbuf);
}

void CRefPlayerDlg::closeStream(u_long sessionId)
{
   CString  s;
   char     nbuff [64];
   CDialog  *dlg;

   s = "RTSP Reference Player - connection to " + m_serverName;
   s += " has been CLOSED";
   SetWindowText( s );
   s = "o rtsp://" + m_serverName + ":";
   sprintf( nbuff, "%d", m_port );
   s += (LPCTSTR) nbuff;
   m_cmdED.SetWindowText( s );
   dlg = getSessionDlg(sessionId);
//   if ( dlg )
//      DestroyWindow( dlg );
   removeSessionDlg(sessionId);
}

void CRefPlayerDlg::writeText(const char *text)
{
    m_textED.SetSel(-1,-1);
    m_textED.ReplaceSel(text);
}

void CRefPlayerDlg::writeText(const char c)
{
    char buf[2];
    buf[0] = c;
    buf[1] = '\0';

    m_textED.SetSel(-1,-1);
    m_textED.ReplaceSel(buf);
}

void CRefPlayerDlg::showDebug(const char *msg, u_long msgLen)
{
    if(msgLen > 0)
    {
	char *msgBuf = new char[msgLen * 3 + 1];
	for(u_long i=0, j=0;i<msgLen;i++, j += 3)
	    sprintf(&msgBuf[j], "%02x ", msg[i] & 0xff);
	msgBuf[j] = '\0';
	writeText(msgBuf);
	writeText("\r\n");
	delete[] msgBuf;
    }
}

void CRefPlayerDlg::initSessionDlg()
{
    for(int i=0;i<MAXSESSIONDLG;i++)
    {
	sessionDlg[i].sessionId = (unsigned long)-1;
	sessionDlg[i].dlg = 0;
    }
}

BOOL CRefPlayerDlg::addSessionDlg(unsigned long sessionId, CDialog *dlg)
{
    for(int i=0;i<MAXSESSIONDLG;i++)
    {
	if(sessionDlg[i].sessionId == (unsigned long)-1)
	{
	    sessionDlg[i].sessionId = sessionId;
	    sessionDlg[i].dlg = dlg;
	    return TRUE;
	}
    }
    return FALSE;
}

void CRefPlayerDlg::removeSessionDlg(unsigned long sessionId)
{
    for(int i=0;i<MAXSESSIONDLG;i++)
    {
	if(sessionDlg[i].sessionId == sessionId)
	{
	    sessionDlg[i].sessionId = (unsigned long)-1;
	    sessionDlg[i].dlg = 0;
	}
    }
}

CDialog *CRefPlayerDlg::getSessionDlg(unsigned long sessionId)
{
    for(int i=0;i<MAXSESSIONDLG;i++)
    {
	if(sessionDlg[i].sessionId == sessionId)
	    return sessionDlg[i].dlg;
    }
    return 0;
}

