/*
 * RTSP (Real Time Streaming Protocol) Reference Implementation
 * 
 *     Player dialog declarations
 *
 *
 *  Copyright (C) 1996 Progressive Networks.
 * 
 *  This program is free software; you can only redistribute it and/or
 *  modify it under the terms of version 2 of the GNU General Public
 *  License as published by the Free Software Foundation.  A copy of this
 *  license may be found in the "LICENSE" directory of this distribution.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY OR REPRESENTATION AS TO OWNERSHIP OR
 *  NON-INFRINGEMENT; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 *  License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 *  Please submit patches, modified versions and derivative works to
 *  Progressive Networks at "rtsp-feedback@prognet.com" or write to
 *  Progressive Networks, 1111 Third Ave. Ste 2900, Seattle, WA 98101, USA.
 *  We will do our best to incorporate your enhancements into the core
 *  distribution.
 *
 *  For more information about this program, please visit the following
 *  URL:
 *     http://www.real.com/prognet/rt 
 *

 *
 * $Id: RefPlayerDlg.h,v 1.12 1997/04/01 17:31:31 robla Exp $
 *
 */

#ifndef _REFPLAYERDLG_H_
#define _REFPLAYERDLG_H_

#include "resource.h"

const int MAXSESSIONDLG = 10;


class CRefPlayerDlg : public CDialog
{
public:
    CRefPlayerDlg(const char *url, CWnd* pParent = NULL);	// standard constructor
    ~CRefPlayerDlg();

    enum { IDD = IDD_REFPLAYER_DIALOG };

    void newSession(u_long session_id);
    void renewPlay( u_long session_id );
    void startPlay( u_long session_id );
    void showDebug(const char *msg, u_long len);
    void closeSession(u_long session_id);
    void hello();
    void displayError(const char *err);
    void displayAlert(const char *msg);
    void streamOutput(u_long session_id, u_short len, const char *buf);
    void streamDone( u_long session_id );
    void resumeStream(u_long session_id);
    void startStream(u_long session_id);
    void pauseStream(u_long session_id);
    void setStreamSeek(u_long session_id, int pos, int range);
    void closeStream(u_long session_id);
    void KillEventTimer();

protected:
    virtual BOOL OnInitDialog();
    afx_msg void OnSysCommand(UINT nID, LPARAM lParam);
    afx_msg void OnPaint();
    afx_msg void OnTimer(UINT);
    afx_msg HCURSOR OnQueryDragIcon();
    afx_msg void OnGoBn();
    afx_msg void OnHelpBtn();
    afx_msg void OnDestroy();

    HICON m_hIcon;

    DECLARE_MESSAGE_MAP()

private:
    struct
    {
	unsigned long sessionId;
	CDialog *dlg;
    } sessionDlg[MAXSESSIONDLG];

    void initSessionDlg();
    BOOL addSessionDlg(unsigned long sessionId, CDialog *dlg);
    void removeSessionDlg(unsigned long sessionId);
    CDialog *getSessionDlg(unsigned long sessionId);
    void writeText(const char *text);
    void writeText(const char c);

    CString m_url;
    CString m_serverName;
    unsigned short m_port;
    CString m_fileName;
    SOCKET m_sock;
    UINT m_timer;

    CEdit m_cmdED;
    CEdit m_textED;
};

#endif	/* _REFPLAYERDLG_H_ */

