/*
 * RTSP (Real Time Streaming Protocol) Reference Implementation
 *
 *     CStreamDlg implementations
 *
 *
 *  Copyright (C) 1996 Progressive Networks.
 * 
 *  This program is free software; you can only redistribute it and/or
 *  modify it under the terms of version 2 of the GNU General Public
 *  License as published by the Free Software Foundation.  A copy of this
 *  license may be found in the "LICENSE" directory of this distribution.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY OR REPRESENTATION AS TO OWNERSHIP OR
 *  NON-INFRINGEMENT; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 *  License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 *  Please submit patches, modified versions and derivative works to
 *  Progressive Networks at "rtsp-feedback@prognet.com" or write to
 *  Progressive Networks, 1111 Third Ave. Ste 2900, Seattle, WA 98101, USA.
 *  We will do our best to incorporate your enhancements into the core
 *  distribution.
 *
 *  For more information about this program, please visit the following
 *  URL:
 *     http://www.real.com/prognet/rt 
 *

 *
 * $Id: StreamDlg.cpp,v 1.8 1997/04/01 17:31:33 robla Exp $
 *
 */

#include "stdafx.h"
#include "socket.h"
#include "RefPlayer.h"
#include "StreamDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CStreamDlg::CStreamDlg(unsigned long streamId, CRefPlayerDlg* refDlg)
    : CDialog(CStreamDlg::IDD, refDlg),
    m_streamId(streamId),
    m_refDlg(refDlg)
{
}


BOOL CStreamDlg::create()
{
    int rc = CDialog::Create(CStreamDlg::IDD, m_refDlg);

    char strbuf[20];
    sprintf(strbuf, "Stream %ld", m_streamId);
    SetWindowText(strbuf);
    return rc;
}

BEGIN_MESSAGE_MAP(CStreamDlg, CDialog)
    ON_WM_HSCROLL()
    ON_BN_CLICKED(ID_PLAY_BN, OnPlayBn)
    ON_BN_CLICKED(ID_STOP_BN, OnStopBn)
END_MESSAGE_MAP()

BOOL CStreamDlg::OnInitDialog()
{
    CDialog::OnInitDialog();
    m_textED.SubclassDlgItem(IDC_TEXT_ED, this);
    m_slider.SubclassDlgItem(IDC_SLIDER, this);
    m_slider.SetRange(0, 100);  /* 0 to 100 */
    m_slider.SetPos( 0 );

    (GetDlgItem(ID_PLAY_BN))->EnableWindow(TRUE);
    (GetDlgItem(ID_STOP_BN))->EnableWindow(FALSE);
    (GetDlgItem(IDC_SLIDER))->EnableWindow(TRUE);

    return TRUE;
}

void CStreamDlg::OnCancel()
{
    DestroyWindow();
}

void CStreamDlg::PostNcDestroy()
{
    delete this;
}


void CStreamDlg::DisablePlayBtn()
{
    (GetDlgItem(ID_PLAY_BN))->EnableWindow(FALSE);
    (GetDlgItem(ID_STOP_BN))->EnableWindow(TRUE);
}

void CStreamDlg::EnablePlayBtn()
{
    (GetDlgItem(ID_PLAY_BN))->EnableWindow(TRUE);
    (GetDlgItem(ID_STOP_BN))->EnableWindow(FALSE);
}

void CStreamDlg::OnHScroll(UINT sbCode, UINT p, CScrollBar *scrollBar)
{
    static int last_pos = 0;

    if(sbCode == SB_THUMBTRACK)
      return;
    int pos = m_slider.GetPos();
    if(pos != last_pos)
    {
      // assumes range of 0 to max
      m_refDlg->setStreamSeek(m_streamId, pos, m_slider.GetRangeMax());
      last_pos = pos;
      DisablePlayBtn();
    }
}

void CStreamDlg::OnPlayBn() 
{
    m_refDlg->resumeStream(m_streamId);
    DisablePlayBtn();
    (GetDlgItem(IDC_SLIDER))->EnableWindow(TRUE);
}

void CStreamDlg::OnStopBn() 
{
    m_refDlg->pauseStream(m_streamId);
    EnablePlayBtn();
    (GetDlgItem(IDC_SLIDER))->EnableWindow(FALSE);
}

void CStreamDlg::RepositionSeekSlider( int pos )
{
    m_slider.SetPos( pos );
    (GetDlgItem(IDC_SLIDER))->EnableWindow(TRUE);
}


void CStreamDlg::OnDestroy()
{
    m_refDlg->closeStream(m_streamId);
}

void CStreamDlg::writeText(const char *text)
{
    m_textED.SetSel(-1,-1);
    m_textED.ReplaceSel(text);
}

void CStreamDlg::closeStream()
{
// the following closed the entire player program and we just want the
// stream dialog to be closed.
//   DestroyWindow();
}
