/*
 * RTSP (Real Time Streaming Protocol) Reference Implementation
 *
 *     Event loop
 * 
 *
 *  Copyright (C) 1996 Progressive Networks.
 * 
 *  This program is free software; you can only redistribute it and/or
 *  modify it under the terms of version 2 of the GNU General Public
 *  License as published by the Free Software Foundation.  A copy of this
 *  license may be found in the "LICENSE" directory of this distribution.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY OR REPRESENTATION AS TO OWNERSHIP OR
 *  NON-INFRINGEMENT; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 *  License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 *  Please submit patches, modified versions and derivative works to
 *  Progressive Networks at "rtsp-feedback@prognet.com" or write to
 *  Progressive Networks, 1111 Third Ave. Ste 2900, Seattle, WA 98101, USA.
 *  We will do our best to incorporate your enhancements into the core
 *  distribution.
 *
 *  For more information about this program, please visit the following
 *  URL:
 *     http://www.real.com/prognet/rt 
 *

 *
 * $Id: eventloop.c,v 1.9 1997/04/01 17:31:35 robla Exp $
 *
 */

#include <assert.h>
#include <sys/types.h>
#include <sys/timeb.h>
#include <time.h>
#include <stdio.h>
#include <string.h>

#include "eventloop.h"
#include "messages.h"
#include "msg_handler.h"
#include "scheduler.h"
#include "socket.h"
#include "util.h"

extern void terminate(int errorcode);
int Quit = 0;

u_long inow;	                   	/* Internal time format (ms)		  */
void (*readers[MAX_FDS])(RTSP_SOCK);    /* Descriptor -> Function table (reading) */
RTSP_SOCK main_fd = 0;	                /* The primary TCP channel's fd		  */
static int  in_eventloop = 0;

void
eventloop_init()
{
    int i;

    for (i = 0; i < MAX_FDS; i++)
	readers[i] = 0;
}

void gettimeofday(struct timeval *tv, void *unused)
{
    struct _timeb timeptr;

    _ftime(&timeptr);

    tv->tv_sec = timeptr.time;
    tv->tv_usec = timeptr.millitm * 1000;
}

void
eventloop()
{
   static struct timeval   start = { 0, 0 };
   struct timeval          t = { 0, 10000 };
   struct timeval          now;
   fd_set                  readset;
   fd_set                  writeset;
   int                     i;

   if ( in_eventloop || Quit )
      return;     /* prevent re-entrant behavior */

   in_eventloop = 1;

   if (!start.tv_sec)
      gettimeofday(&start, 0);

   FD_ZERO(&readset);
   FD_ZERO(&writeset);

   if (main_fd)
   {
     assert( main_fd < FD_SETSIZE );
     if ( main_fd >= FD_SETSIZE )
     {
        printf( "main_fd assigned exceeded FD_SETSIZE limit.\n" );
        terminate( 1 );
     }
     FD_SET(main_fd, &readset);
   }

   for (i = 0; i < MAX_FDS; i++)
   {
     if (readers[i])
        FD_SET(i, &readset);
   }

   if (main_fd && io_write_pending())
     FD_SET(main_fd, &writeset);

   select(FD_SETSIZE, &readset, &writeset, 0, &t);

   gettimeofday(&now, 0);
   inow = ((now.tv_sec - start.tv_sec) * 1000) + (now.tv_usec / 1000);

   schedule_execute(inow);

   if (main_fd && FD_ISSET(main_fd, &readset))
   {
     io_read(main_fd);
     msg_handler();
   }

   if ( !Quit )
   {
      if (main_fd && FD_ISSET(main_fd, &writeset))
        io_write(main_fd);

      for (i = 0; i < MAX_FDS; i++)
      {
        if (FD_ISSET(i, &readset) && readers[i])
           readers[i](i);
      }
   }

   in_eventloop = 0;
}
